from dectate import Action


class HomepageWidgetAction(Action):
    """ Register a cronjob. """

    config = {
        'homepage_widget_registry': dict
    }

    def __init__(self, tag):
        self.tag = tag

    def identifier(self, homepage_widget_registry):
        return self.tag

    def perform(self, func, homepage_widget_registry):
        widget = func()
        widget.tag = self.tag  # keep redundantly for ease of access

        homepage_widget_registry[self.tag] = widget


class ExportAction(Action):
    """ Register an export. """

    config = {
        'export_registry': dict
    }

    def __init__(self, id, **kwargs):
        self.id = id
        self.kwargs = kwargs
        self.kwargs['id'] = id

    def identifier(self, export_registry):
        return self.id

    def perform(self, cls, export_registry):
        export_registry[self.id] = cls(**self.kwargs)


class UserlinkAction(Action):
    """ Registers a user link group. """

    config = {
        'linkgroup_registry': list
    }

    counter = iter(range(1, 123456789))

    def __init__(self):
        self.name = next(self.counter)

    def identifier(self, linkgroup_registry):
        return self.name

    def perform(self, func, linkgroup_registry):
        linkgroup_registry.append(func)


class DirectorySearchWidgetAction(Action):
    """ Registers a directory search widget. """

    config = {
        'directory_search_widget_registry': dict
    }

    def __init__(self, name):
        self.name = name

    def identifier(self, directory_search_widget_registry):
        return self.name

    def perform(self, cls, directory_search_widget_registry):
        cls.name = self.name

        assert hasattr(cls, 'html')
        assert hasattr(cls, 'adapt')

        directory_search_widget_registry[self.name] = cls



from onegov.org import _
from onegov.org.forms.generic import DateRangeForm, ExportForm
from onegov.form import merge_forms
from onegov.form.fields import MultiCheckboxField
from wtforms import RadioField
from wtforms import validators
from wtforms.fields.html5 import DateField


class FormSubmissionsExport(merge_forms(DateRangeForm, ExportForm)):

    selection = RadioField(
        label=_("Selection"),
        default='date',
        choices=[
            ('date', _("By date")),
            ('window', _("By registration window"))
        ]
    )

    registration_window = MultiCheckboxField(
        label=_("Registration Window"),
        choices=None,
        depends_on=('selection', 'window')
    )

    start = DateField(
        label=_("Start"),
        validators=[validators.InputRequired()],
        depends_on=('selection', 'date')
    )

    end = DateField(
        label=_("End"),
        validators=[validators.InputRequired()],
        depends_on=('selection', 'date')
    )

    def on_request(self):
        if self.registration_window.choices is None:
            self.load_registration_windows()

            if not self.registration_window.choices:
                self.hide(self.selection)

    def load_registration_windows(self):
        # XXX circular import - also, a layout class just to get a date format?
        from onegov.org.layout import DefaultLayout

        layout = DefaultLayout(self.model, self.request)

        def key(window):
            return window.id.hex

        def title(window):
            return layout.format_date_range(window.start, window.end)

        def choice(window):
            return key(window), title(window)

        windows = self.model.registration_windows
        windows.sort(key=lambda r: r.start)

        self.registration_window.choices = tuple(choice(w) for w in windows)
